#include "Building.h"
#include "Planet.h"
#include "ParticleSystem.h"

const float Building::s_totalGrowTime = 5000.0f;

Building::Building()
{
    m_growTime = 0.0f;
    m_top      = (rand() % 2) != 0;
}

void Building::Update(Planet& planet, float deltaTime)
{

    m_growTime = min(m_growTime + deltaTime, s_totalGrowTime);

    if (planet.GetIsUnderwater(m_x))
    {
        Kill(planet);
    }

}

void Building::Render(const Planet& planet, float time) const
{

    float scale = m_growTime / s_totalGrowTime;

    glBegin(GL_QUADS);

    glColor3f(0.0f, 0.0f, 0.0f);
    glVertex2f(-20.0f, -50.0f);
    glVertex2f(-20.0f,  80.0f * scale);
    glVertex2f( 20.0f,  80.0f * scale);
    glVertex2f( 20.0f, -50.0f);

    glColor3f(1.0f, 1.0f, 1.0f);

    for (int y = 0; y < 4; ++y)
    {
        for (int x = 0; x < 2; ++x)
        {

            float u = -20.0f + 6.0f + x * 18.0f;
            float v =   2.0f + 6.0f + y * 18.0f;

            glVertex2f(u,         v * scale);
            glVertex2f(u + 12.0f, v * scale);
            glVertex2f(u + 12.0f, (v + 12.0f) * scale);
            glVertex2f(u,         (v + 12.0f) * scale);

        }
    }

    glEnd();

    if (m_top)
    {

        // Draw a top to the building.

        glColor3f(0.0f, 0.0f, 0.0f);
        glBegin(GL_TRIANGLES);
        glVertex2f(-15.0f,  80.0f * scale);
        glVertex2f(  0.0f, 100.0f * scale);
        glVertex2f( 15.0f,  80.0f * scale);
        glEnd();

        glLineWidth(2.0f);
        glBegin(GL_LINES);
        glVertex2f(0.0f, 100.0f * scale);
        glVertex2f(0.0f, 130.0f * scale);
        glEnd();

    }

}

bool Building::GetIsAlive() const
{
    return m_alive;
}

void Building::Kill(Planet& planet)
{

    m_alive = false;

    const float height = 20.0f;
    const float radius = 20.0f;

    ParticleSystem* effect = new ParticleSystem;
    effect->InitializeCircle(Color(0.0f, 0.0f, 0.0f), planet.GetPoint(m_x, height), radius, 0.1f, 1000.0f, 100);

    planet.AddEffect(effect);

}

