#include "Color.h"
#include <math.h>

Color::Color()
{
}
    
Color::Color(float _r, float _g, float _b)
    : r(_r), g(_g), b(_b)
{
}

void Color::SetHSV(float h, float s, float v)
{

	int i;
	float f, p, q, t;

	if( s == 0 ) {
		// achromatic (grey)
		r = g = b = v;
		return;
	}

	h *= 6.0f;			// sector 0 to 5
	i = static_cast<int>(floorf( h ));
	f = h - i;			// factorial part of h
	p = v * ( 1 - s );
	q = v * ( 1 - s * f );
	t = v * ( 1 - s * ( 1 - f ) );

	switch( i ) {
	case 0:
		r = v;
		g = t;
		b = p;
		break;
	case 1:
		r = q;
		g = v;
		b = p;
		break;
    case 2:
		r = p;
		g = v;
		b = t;
		break;
	case 3:
		r = p;
		g = q;
		b = v;
		break;
	case 4:
		r = t;
		g = p;
	    b = v;
		break;
	default:
		r = v;
		g = p;
		b = q;
		break;
	}

}

const Color& GetPrettyColor(int n)
{

    static const Color color[] =
        {
            Color(1.0f, 0.5f, 0.5f),
            Color(0.5f, 1.0f, 0.5f),
            Color(0.5f, 0.5f, 1.0f),
            Color(1.0f, 1.0f, 0.5f),
            Color(1.0f, 0.5f, 1.0f),
            Color(0.5f, 1.0f, 1.0f),
        };

    return color[n % (sizeof(color) / sizeof(Color))];

}