#include "Meteor.h"
#include "Math.h"
#include "OpenGL.h"
#include "Planet.h"
#include "Meteorite.h"
#include "Noise.h"

void Meteor::Initialize()
{
    m_incomingAngle = Random(pi * 0.3f, 2.0f * pi);
    m_distance      = 1000.0f;
    m_speed         = 0.3f;
    m_alive         = true;
    m_resource      = 1.0f;
}

void Meteor::Render(const Planet& planet)
{

    // Use the depth test so that we don't write the same alpha blended surface twice.
    glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

    Vec2 origin;

    origin.x = cosf(m_incomingAngle) * m_distance;
    origin.y = sinf(m_incomingAngle) * m_distance;

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glTranslatef(origin.x, origin.y, 0.0f);

    glColor4f(00.0f, 0.0f, 0.0f, 0.5f);
    DrawFilledCircle(Vec2(0.0f, 0.0f), 15.0f, 32);

    glPopMatrix();

    glDisable(GL_DEPTH_TEST);

}

void Meteor::Update(Planet& planet, float deltaTime)
{
    
    m_distance -= m_speed * deltaTime;

    float x = (m_incomingAngle + planet.GetAngle()) / (2.0f * pi);

    if (m_distance < planet.GetLandHeight(x))
    {

        // Create a new meteor.
        Meteorite* meteorite = new Meteorite;
        meteorite->SetPosition(x);
        planet.AddMeteorite(meteorite);

        m_alive = false;

    }

}

bool Meteor::GetIsAlive() const
{
    return m_alive;
}

float Meteor::GetResource() const
{
    return m_resource;
}

void Meteor::SetResource(float resource)
{
    m_resource = resource;
}
