#include "Nuke.h"
#include "Vec2.h"
#include "Planet.h"
#include "OpenGL.h"

void Nuke::Initialize(float xStart, float xEnd, float speed)
{
    m_xStart    = xStart;
    m_xEnd      = xEnd;
    m_t         = 0.0f;
    m_speed     = speed;
}

void Nuke::Render(const Planet& planet) const
{

    // Compute the cubic Bezier curve.

    const float height = 300.0f;

    Vec2 p0 = planet.GetPoint(m_xStart, 0.0f);
    Vec2 p1 = planet.GetPoint(m_xStart, height);
    Vec2 p2 = planet.GetPoint(m_xEnd,   height);
    Vec2 p3 = planet.GetPoint(m_xEnd,   0.0f);

    glColor3f(0.0f, 0.0f, 0.0f);

    float t  = m_t;
    float t2 = t * t;
    float t3 = t * t2;

    float s  = 1.0f - t;
    float s2 = s * s;
    float s3 = s * s2;

    Vec2 origin = p0 * s3 + p1 * 3.0f * t * s2 + p2 * 3.0f * t2 * s + p3 * t3;  
    DrawFilledCircle(origin, 10.0f);

    glLineWidth(1.0f);

    glColor3f(1.0f, 1.0f, 1.0f);
    DrawCircle(origin, 10.0f);

}

void Nuke::Update(Planet& planet, float deltaTime)
{

    m_t += m_speed * deltaTime;

    if (m_t > 1.0f)
    {
        // Detonate the nuke.
        planet.CreateExplosion(m_xEnd);
    }

}

bool Nuke::GetIsAlive() const
{
    return m_t < 1.0f;
}