#include "OpenGL.h"
#include "Math.h"

void glPoint(const Vec2& point)
{
    glVertex2f(point.x, point.y);
}

void glColor(const Color& color)
{
    glColor3f(color.r, color.g, color.b);
}

void DrawFilledCircle(const Vec2& point, float radius, int numSegments)
{

    const float deltaAngle  = 2.0f * pi / (numSegments - 1.0f);

    float angle = 0.0f;

    glBegin(GL_TRIANGLE_FAN);

    for (int i = 0; i < numSegments; ++i)
    {
        glVertex2f(point.x + cosf(angle) * radius, point.y + sinf(angle) * radius);
        angle += deltaAngle;
    }

    glEnd();

}

void DrawFilledEllipse(const Vec2& point, float xRadius, float yRadius, int numSegments)
{

    const float deltaAngle  = 2.0f * pi / (numSegments - 1.0f);

    float angle = 0.0f;

    glBegin(GL_TRIANGLE_FAN);

    for (int i = 0; i < numSegments; ++i)
    {
        glVertex2f(point.x + cosf(angle) * xRadius, point.y + sinf(angle) * yRadius);
        angle += deltaAngle;
    }

    glEnd();

}


void DrawCircle(const Vec2& point, float radius, int numSegments)
{

    const float deltaAngle  = 2.0f * pi / (numSegments - 1.0f);

    float angle = 0.0f;

    glBegin(GL_LINE_STRIP);

    for (int i = 0; i < numSegments; ++i)
    {
        glVertex2f(point.x + cosf(angle) * radius, point.y + sinf(angle) * radius);
        angle += deltaAngle;
    }

    glEnd();

}