#include "ParticleSystem.h"
#include "Math.h"
#include "OpenGL.h"

void ParticleSystem::InitializeCircle(const Color& color, const Vec2& origin, float radius, float speed, float life, int numParticles)
{

    m_maxLife       = life;
    m_color         = color;
    m_numParticles  = min(s_maxParticles, numParticles);

    for (int i = 0; i < m_numParticles; ++i)
    {

        m_particle[i].life = life;

        while (1)
        {
         
            m_particle[i].velocity.x = Random(-radius, radius);
            m_particle[i].velocity.y = Random(-radius, radius);

            float d = m_particle[i].velocity.GetLength();

            if (d <= radius)
            {
                m_particle[i].position = m_particle[i].velocity;
                m_particle[i].position.x += origin.x;
                m_particle[i].position.y += origin.y;
                m_particle[i].velocity.x *= speed / d;
                m_particle[i].velocity.y *= speed / d;
                break;
            }

        }

    }

}

void ParticleSystem::Render(const Planet& planet) const
{

    glBegin(GL_QUADS);
    glColor(m_color);

    for (int i = 0; i < m_numParticles; ++i)
    {

        // Shrink the particle as it gets older.
        float size = 1.5f * m_particle[i].life / m_maxLife;

        glVertex2f(m_particle[i].position.x - size, m_particle[i].position.y - size);
        glVertex2f(m_particle[i].position.x + size, m_particle[i].position.y - size);
        glVertex2f(m_particle[i].position.x + size, m_particle[i].position.y + size);
        glVertex2f(m_particle[i].position.x - size, m_particle[i].position.y + size);

    }

    glEnd();

}

void ParticleSystem::Update(Planet& planet, float deltaTime)
{
    
    for (int i = 0; i < m_numParticles; ++i)
    {
        
        m_particle[i].position.x += m_particle[i].velocity.x * deltaTime;
        m_particle[i].position.y += m_particle[i].velocity.y * deltaTime;

        m_particle[i].life -= deltaTime;

        if (m_particle[i].life < 0.0f)
        {
            // Particle is dead, so move it to the end.
            m_particle[i] = m_particle[m_numParticles - 1];
            --m_numParticles;
            --i;
        }

    }

}

bool ParticleSystem::GetIsAlive() const
{
    return m_numParticles > 0;
}
