#include "Plant.h"
#include "Math.h"
#include "OpenGL.h"
#include "Planet.h"
#include "ParticleSystem.h"

const float Plant::s_totalGrowTime = 1000.0f;

Plant::Plant()
{
    m_growTime  = 0.0f;
    m_scale     = Random(0.5f, 1.5f);
    m_alive     = true;
    m_state     = State_Growing;
}

void Plant::Update(Planet& planet, float deltaTime)
{

    const float dieRate = 0.1f;
 
    // Check if we're on the shady side of the planet.

    if (planet.GetIsSunny(m_x))
    {
        m_state = State_Growing;
    }
    else
    {
        m_state = State_Dying;
    }

    if (m_state == State_Growing)
    {
        m_growTime = min(m_growTime + deltaTime, s_totalGrowTime);
    }
    else if (m_state == State_Dying)
    {

        m_growTime -= deltaTime * dieRate;
    
        if (m_growTime < 0)
        {
            m_alive = false;
        }
    
    }

    // Check if we should kill the guy (if we're under water).

    if (!planet.GetCanSupportPlants(m_x))
    {
        Kill(planet);
    }

}

void Plant::Render(const Planet& planet, float time) const
{

    float growthFraction = min(m_growTime / s_totalGrowTime, 1.0f);

    // Use the depth test so that we don't write the same alpha blended surface twice.
    glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

    // Draw a pine tree-like structure.

    glColor4f(0.0f, 0.0f, 0.0f, 0.5f);

    float s = growthFraction * m_scale;

    // Draw the cone part.
    glBegin(GL_TRIANGLES);
    glVertex2f(  0.0f * s, 50.0f * s);
    glVertex2f( 10.0f * s, 10.0f * s);
    glVertex2f(-10.0f * s, 10.0f * s);
    glEnd();

    // Draw the trunk (we extend it below the ground so it looks right on a hill).
    glBegin(GL_QUADS);
    glVertex2f( -3.0f * s,   9.0f * s);
    glVertex2f(  3.0f * s,   9.0f * s);
    glVertex2f(  3.0f * s, -10.0f * s);
    glVertex2f( -3.0f * s, -10.0f * s);
    glEnd();

    glDisable(GL_DEPTH_TEST);

}

bool Plant::GetIsAlive() const
{
    return m_alive;
}

void Plant::Kill(Planet& planet)
{

    m_alive = false;

    ParticleSystem* effect = new ParticleSystem;
    effect->InitializeCircle(Color(0.0f, 0.0f, 0.0f), planet.GetPoint(m_x, 5.0f), 10.0f, 0.1f, 1000.0f);

    planet.AddEffect(effect);

}