#include "Sparkle.h"
#include "OpenGL.h"
#include "Math.h"

void Sparkle::Initialize(const Vec2& origin, float life)
{
    m_origin    = origin;
    m_life      = 0.0f;
    m_maxLife   = life;
}

void Sparkle::Render(const Planet& planet) const
{

    float angle = m_life * 0.003f;
    float scale = 10.0f * Bump(1.0f - (m_life / m_maxLife) * 2.0f);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    glTranslatef(m_origin.x, m_origin.y, 0.0f);
    glRotatef(GetDegrees(angle), 0.0f, 0.0f, 1.0f);

    glColor3f(1.0f, 1.0f, 1.0f);
    glBegin(GL_QUADS);

    glPoint(Vec2(-1.00f,  0.00f) * scale);
    glPoint(Vec2(-0.25f,  0.25f) * scale);
    glPoint(Vec2( 0.00f,  0.00f) * scale);
    glPoint(Vec2(-0.25f, -0.25f) * scale);

    glPoint(Vec2( 1.00f,  0.00f) * scale);
    glPoint(Vec2( 0.25f,  0.25f) * scale);
    glPoint(Vec2( 0.00f,  0.00f) * scale);
    glPoint(Vec2( 0.25f, -0.25f) * scale);

    glPoint(Vec2( 0.00f, 1.00f) * scale);
    glPoint(Vec2( 0.25f, 0.25f) * scale);
    glPoint(Vec2( 0.00f, 0.00f) * scale);
    glPoint(Vec2(-0.25f, 0.25f) * scale);

    glPoint(Vec2( 0.00f, -1.00f) * scale);
    glPoint(Vec2( 0.25f, -0.25f) * scale);
    glPoint(Vec2( 0.00f,  0.00f) * scale);
    glPoint(Vec2(-0.25f, -0.25f) * scale);

    glEnd();

    glLineWidth(1.0f);
    glColor3f(0.0f, 0.0f, 0.0f);

    glBegin(GL_LINE_LOOP);

    glPoint(Vec2(-1.00f,  0.00f) * scale);
    glPoint(Vec2(-0.25f,  0.25f) * scale);
    glPoint(Vec2( 0.00f,  1.00f) * scale);
    glPoint(Vec2( 0.25f,  0.25f) * scale);
    glPoint(Vec2( 1.00f,  0.00f) * scale);
    glPoint(Vec2( 0.25f, -0.25f) * scale);
    glPoint(Vec2( 0.00f, -1.00f) * scale);
    glPoint(Vec2(-0.25f, -0.25f) * scale);

    glEnd();
    
    glPopMatrix();

}

void Sparkle::Update(Planet& planet, float deltaTime)
{
    m_life += deltaTime;
}

bool Sparkle::GetIsAlive() const
{
    return m_life < m_maxLife;
}
