#ifndef TIMER_H
#define TIMER_H

#include <windows.h>

/**
 * Timer class.
 */
class Timer
{

public:

    /**
     * Constructor.
     */
    Timer();

    /**
     * Resets the timer. This will cause subsequent calls to
     * GetTotalElapsedTime to return the number of milliseconds since Reset
     * was called.
     */
    void Reset();

    /**
     * Polls the system clock and updates the time stored in the timer class.
     */
    void Update();

    /**
     * Returns the time which has elapsed between the two previous two calls to
     * Update.  Returns the time in milliseconds between the past two times the
     * timer was updated.
     */
    float GetElapsedTime() const;

    /**
     * Returns the time which has elapsed between when the timer was reset and
     * last time Update was called.
     */
    float GetTotalElapsedTime() const;

private:

    LARGE_INTEGER   m_frequency;

    LARGE_INTEGER   m_resetTime;
    LARGE_INTEGER   m_lastTime;
    LARGE_INTEGER   m_currTime;

};

#endif